﻿using System;
using System.Globalization;
using System.IO;
using System.Text;
using System.Windows.Forms;

using System.Speech.Synthesis;
using System.Speech.AudioFormat;
//using Microsoft.Speech.Synthesis;
//using Microsoft.Speech.AudioFormat;

namespace SpeechTest
{
    partial class MainForm : Form
    {
        SpeechSynthesizer tts;

        public MainForm()
        {
            InitializeComponent();

            tts = new SpeechSynthesizer();
            //tts.SelectVoiceByHints(VoiceGender.Female, VoiceAge.Adult, 0, CultureInfo.CurrentCulture);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            int selectVoice = -1;
            foreach (InstalledVoice voice in tts.GetInstalledVoices())
            {
                int i = voicesComboBox.Items.Add(voice.VoiceInfo.Name);
                if (selectVoice < 0 && voice.VoiceInfo.Culture.Equals(CultureInfo.CurrentCulture))
                    selectVoice = i;
            }
            if (selectVoice >= 0)
                voicesComboBox.SelectedIndex = selectVoice;
        }

        private void sayButton_Click(object sender, EventArgs e)
        {
            tts.Volume = (int)volumeUpDown.Value;
            tts.Rate = (int)rateUpDown.Value;
            tts.SelectVoice(voicesComboBox.SelectedItem.ToString());
            tts.SetOutputToDefaultAudioDevice();
            tts.Speak(testTextBox.Text);
            tts.SetOutputToNull();
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            using (SaveFileDialog dlg = new SaveFileDialog())
            {
                dlg.Title = "Save Audio File";
                dlg.DefaultExt = "wav";
                dlg.Filter = "Wave Sound (*.wav)|*.wav";

                if (dlg.ShowDialog(this) != DialogResult.OK)
                    return;

                Cursor = Cursors.WaitCursor;
                Refresh();
                Application.DoEvents();

                tts.Volume = (int)volumeUpDown.Value;
                tts.Rate = (int)rateUpDown.Value;
                tts.SelectVoice(voicesComboBox.SelectedItem.ToString());
                tts.SetOutputToWaveFile(dlg.FileName,
                    tts.Voice.SupportedAudioFormats.Count > 0 ? tts.Voice.SupportedAudioFormats[0] : new SpeechAudioFormatInfo(22050, AudioBitsPerSample.Eight, AudioChannel.Mono));
                tts.Speak(testTextBox.Text);
                tts.SetOutputToNull();

                Cursor = Cursors.Default;
            }
        }

        private void batchConvertButton_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog sourceDlg = new OpenFileDialog())
            using (FolderBrowserDialog targetDlg = new FolderBrowserDialog())
            {
                sourceDlg.Title = "Open Text Files to Convert";
                sourceDlg.DefaultExt = "txt";
                sourceDlg.Filter = "Text Files (*.txt)|*.txt|All Files|*.*";
                sourceDlg.Multiselect = true;

                if (sourceDlg.ShowDialog(this) != DialogResult.OK)
                    return;

                targetDlg.Description = "Select Folder to Save WAV Files";
                targetDlg.SelectedPath = Path.GetDirectoryName(sourceDlg.FileName);

                if (targetDlg.ShowDialog(this) != DialogResult.OK)
                    return;

                tts.Volume = (int)volumeUpDown.Value;
                tts.Rate = (int)rateUpDown.Value;
                tts.SelectVoice(voicesComboBox.SelectedItem.ToString());
                SpeechAudioFormatInfo waveFormat;
                if (tts.Voice.SupportedAudioFormats.Count > 0)
                    waveFormat = tts.Voice.SupportedAudioFormats[0];
                else
                    waveFormat = new SpeechAudioFormatInfo(22050, AudioBitsPerSample.Eight, AudioChannel.Mono);

                Cursor = Cursors.WaitCursor;
                Refresh();
                Application.DoEvents();

                foreach (string textFile in sourceDlg.FileNames)
                {
                    string waveFile = Path.Combine(targetDlg.SelectedPath, Path.GetFileNameWithoutExtension(textFile) + ".wav");
                    tts.SetOutputToWaveFile(waveFile, waveFormat);
                    using (StreamReader text = new StreamReader(textFile, Encoding.Default, true))
                    {
                        tts.Speak(text.ReadToEnd());
                    }
                    tts.SetOutputToNull();
                }

                Cursor = Cursors.Default;
                MessageBox.Show(this, string.Format("{0} WAV files written in {1}.", sourceDlg.FileNames.Length, targetDlg.SelectedPath),
                    Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }
    }
}
